;; Copyright 2010, 2011, 2013, 2014, 2015 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;; 	- directory name in -files      ???



;;-----------------------------------------------------------------------------
;; run on miscbits files

(progn
  (save-some-buffers)
  (let ((file-list (file-expand-wildcards "../*.el")))
    (setq file-list (mapcar (lambda (filename)
                              (let ((basename (file-name-nondirectory filename)))
                                (cond ((file-exists-p (concat "~/lisp/" basename))
                                       (concat "~/lisp/" basename))
                                      ((file-exists-p (concat "../wip/" basename))
                                       (concat "../wip/" basename))
                                      (t filename))))
                            file-list))
    (compile (concat "emacs-checkdoc-batch "
                     (mapconcat 'shell-quote-argument file-list " ")))))


;;-----------------------------------------------------------------------------
;; run on all my elisp

(progn
  (save-some-buffers)
  (let ((file-list (append (file-expand-wildcards "~/lisp/*.el")
                           (file-expand-wildcards "../wip/*.el")
                           (file-expand-wildcards "../*.el"))))
    (setq file-list (delete-duplicates file-list :key 'file-name-base))
    (setq file-list (delete-if (lambda (filename) (string-match "\\`d-" (file-name-base filename)))
                               file-list))
    (compile (concat "emacs-checkdoc-batch "
                     (mapconcat 'shell-quote-argument file-list " ")))))


;;-----------------------------------------------------------------------------

(add-to-list 'load-path "..")
(require 'checkdoc-batch)
(compile "emacs-checkdoc-batch d-checkdoc-batch-sample.el")

(progn
  (find-file "d-checkdoc-batch-sample.el")
  (checkdoc-batch))

(progn
  (unless (file-exists-p "/tmp/foo.el")
    (copy-file "d-checkdoc-batch-sample.el" "/tmp/foo.el"))
  (find-file "/tmp/foo.el")
  (checkdoc-batch))

(checkdoc-batch-file "/tmp/miscbits-el-finder-inf.el")



;;-----------------------------------------------------------------------------

  (checkdoc-batch--with-ad-enable-advice
   checkdoc-autofix-ask-replace around checkdoc-batch
   (checkdoc-batch--with-ad-enable-advice
    checkdoc-y-or-n-p around checkdoc-batch
    (checkdoc-batch--with-ad-enable-advice
     checkdoc-recursive-edit around checkdoc-batch
     (with-temp-buffer
       (insert-file-contents filename)
       (checkdoc))))))


(defmacro checkdoc-batch--with-ad-enable-advice (func class name &rest body)
  `(let* ((checkdoc-batch-with-ad-enable-advice--find
           (ad-find-advice ',func ',class ',name))
          (checkdoc-batch-with-ad-enable-advice--change
           (and checkdoc-batch-with-ad-enable-advice--find
                (not (ad-advice-enabled checkdoc-batch-with-ad-enable-advice--find)))))
     (unwind-protect
         (progn
           (when checkdoc-batch-with-ad-enable-advice--change
             (ad-enable-advice ',func ',class ',name)
             (ad-activate      ',func))
           ,@body)
       (when checkdoc-batch-with-ad-enable-advice--change
         (ad-disable-advice ',func ',class ',name)
         (ad-activate       ',func)))))
(put 'checkdoc-batch--with-ad-enable-advice 'lisp-indent-function 3)

(defun checkdoc-batch-linenum (pos)
  (save-excursion
    (goto-char pos)
    (1+ (count-lines (point-min) (line-beginning-position)))))

(defadvice checkdoc-display-status-buffer (around checkdoc-batch)
  ad-do-it
  (message "checkdoc-display-status-buffer: %S\n" (buffer-list))
  (require 'cl)
)

  (let ((buffer (get-file-buffer filename)))
    (when buffer
      (save-some-buffers nil (lambda ()
                               (eq buffer (current-buffer))))))



;; with fakery to apply all changes there's nothing interesting in the status

  (let ((buffer (get-buffer " *Checkdoc Status*")))
    (when buffer
      (with-current-buffer buffer
        (message "%s:\n%s"
                 (buffer-name)
                 (buffer-string)))))

;;-----------------------------------------------------------------------------

(let (x)
  ;;   (with-temp-buffer
  ;;     (insert-file-contents "/etc/motd" t)
  (find-file "/etc/motd")
  (let ((motd-buffer (current-buffer)))

    (with-temp-buffer
      (let ((temp-buffer (current-buffer)))
        (let ((buffer-file-name "foo"))

          (setq x buffer-file-name)
          (kill-buffer temp-buffer)
          (set-buffer motd-buffer)
          (current-buffer)
          buffer-file-name))))
  x)


;;-----------------------------------------------------------------------------
;; compile

(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-byte-compile))
(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-show-autoloads))


;;-----------------------------------------------------------------------------
;; autorevert

auto-revert-interval
