;; Copyright 2013, 2014 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;;-----------------------------------------------------------------------------

(progn
  (defun my-grep-form (form)
    (when (and (eq 'defcustom (car-safe form))
               (listp (cdr-safe form)))

      (let ((var       (nth 1 form))
            (default   (nth 2 form))
            (docstring (nth 3 form)))
        (when (null default)
          (setq form (nthcdr 4 form))
          (let ((type (plist-get form :type)))
            (setq type (my-unquote type))
            (when (or ;; t
                   (memq type '(string regexp file character color))
                   (memq (car-safe type) '(string regexp file character color)))

              (let ((line   (count-lines (point-min) (point)))
                    (column (current-column)))
                (with-current-buffer (get-buffer-create "*mygrep*")
                  (insert (format "%s:%s:%s:\n %S\n default %S but :type %S\n\n"
                                  filename line column
                                  var default type))
                  (redisplay t)))
              ))))))

  (defun my-check-buffer ()
    (save-excursion
      (goto-char (point-min))
      (catch 'return
        (while t
          (let ((pending (condition-case err
                             (list (read (current-buffer)))
                           (end-of-file
                            (throw 'return nil))
                           (error
                            (let ((line   (count-lines (point-min) (point)))
                                  (column (current-column)))
                              (with-current-buffer (get-buffer-create "*mygrep*")
                                (insert (format "%s:%d:%d: error %S\n"
                                                filename line column
                                                (error-message-string err)))))
                            (redisplay t)
                            (throw 'return nil)))))
            (while pending
              (let ((obj (pop pending)))
                (cond ((consp obj)
                       (my-grep-form obj)
                       (while (consp obj)
                         (push (pop obj) pending))
                       (push obj pending))
                      ((vectorp obj)
                       (dotimes (i (length obj))
                         (push (elt obj i) pending)))))))))))

  (defun my-check-file (filename)
    ;; (with-current-buffer (get-buffer-create "*mygrep*")
    ;;   (insert (format "%s\n" filename))
    ;;   (redisplay t))

    (with-temp-buffer
      (insert-file-contents filename)
      (my-check-buffer)))

  (defun my-unquote (obj)
    "Extract value from a quote form.
If OBJ is a list (quote X) then return X, otherwise return OBJ."
    (if (and (listp obj)
             (eq 'quote (car obj)))
        (cadr obj)
      obj))

  (switch-to-buffer "*mygrep*")
  (fundamental-mode)
  (setq buffer-read-only nil)
  (erase-buffer)
  (let ((count 0)
        (emacs-lisp-mode-hook nil))
    (dolist (filename (append
                       ;; '("/usr/share/emacs/24.3/lisp/minibuffer.el.gz")

                       (file-expand-wildcards "/usr/share/emacs/site-lisp/*.el")
                       (file-expand-wildcards "/usr/share/emacs/site-lisp/*/*.el")
                       ;; (file-expand-wildcards "/usr/share/emacs/24.3/lisp/*.el.gz")
                       ;; (file-expand-wildcards "/usr/share/emacs/24.3/lisp/*/*.el.gz")

                       ;; (my-locate-list "*.el")
                       ;; (my-locate-list "*.el.gz")
                       ))
      (my-check-file filename)))
  (switch-to-buffer "*mygrep*")
  (compilation-mode))





;;-----------------------------------------------------------------------------

(progn
  (unless (featurep 'my-load-all)
    (load (expand-file-name "my-load-all.el")))
  (my-load-all--custom))

(let ((type-to-predicate '((string    . stringp)
                           (regexp    . my-regexp-p)
                           (file      . my-file-p)
                           ((file :must-match t) . my-file-must-match-p)
                           (directory . stringp)

                           ;; any non-nil is good enough, but see if some
                           ;; bools are meant to be other
                           (boolean   . booleanp)

                           (integer   . integerp)
                           (number    . numberp)
                           (float     . floatp)
                           (character . characterp)
                           (symbol    . symbolp)
                           (function  . functionp)
                           (variable  . symbolp)
                           (face      . facep)
                           (coding-system . coding-system-p)
                           (color     . stringp)
                           ))
      pred
      lst)

  (defun my-file-must-match-p (obj)
    (and (stringp obj)
         (file-exists-p obj)
         (not (file-directory-p obj))))
  (defun my-file-p (obj)
    (and (stringp obj)
         (or (equal "" obj)
             (not (file-directory-p obj)))))
  (defun my-regexp-p (obj)
    (and (stringp obj)
         (condition-case nil
             (progn (string-match obj "") t)
           (invalid-regexp nil))))

  (mapatoms (lambda (symbol)
              (when (and (boundp symbol)
                         (setq pred (cdr (assq (get symbol 'custom-type)
                                               type-to-predicate)))
                         (not (funcall pred (symbol-value symbol))))
                (push symbol lst))))

  (setq lst (sort* lst 'string-lessp :key 'symbol-name))

  (dolist (symbol lst)
    (insert (format "%S   %S %S\n"
                    symbol
                    (get symbol 'custom-type)
                    (symbol-value symbol)))))
ac-candidate-limit   integer nil
ac-trigger-key   string nil
bibtex-file-path   string nil (require 'bibtex)
bibtex-string-file-path   string nil
bookmark-search-delay   integer 0.2
brltty-default-port   integer (4101 4102 4103 4104 35751)
cperl-pod-here-fontify   boolean (featurep (quote font-lock))
debian-bug-use-From-address   boolean "user42_kevin@yahoo.com.au"  (require 'debian-bug)
dict-buffer-coding-system   string nil
eldav-no-proxy-url-regexp   regexp nil
eldav-proxy   string nil
erc-notifications-icon   file nil
ex-unix-type-shell   boolean 5
folding-font-lock-begin-mark   face font-lock-reference-face
folding-font-lock-end-mark   face font-lock-reference-face
folding-internal-margins   boolean 1   (require 'folding)
gnus-async-post-fetch-function   function nil
gnus-blocked-images   regexp gnus-block-private-groups
gnus-gravatar-size   integer nil
gnus-html-image-cache-ttl   integer (9 14976)
gnus-notifications-timeout   integer nil
gnus-selected-tree-face   face modeline
idlwave-help-browser-generic-args   string nil
idlwave-help-browser-generic-program   string nil
lpr-add-switches   boolean (gnu/linux)
maplev-mint-include-dir   string nil
matlab-cont-requires-ellipsis   integer t (require 'matlab)
message-user-organization-file   file nil
org-agenda-auto-exclude-function   function nil
org-agenda-day-face-function   function nil
org-agenda-export-html-style   string nil
org-agenda-ndays   integer nil
org-bibtex-prefix   string nil
org-clock-heading-function   function nil
org-columns-modify-value-for-display-function   function nil
org-export-docbook-doctype   string nil
org-export-docbook-xsl-fo-proc-command   string nil
org-export-docbook-xslt-proc-command   string nil
org-export-docbook-xslt-stylesheet   string nil
org-export-html-postamble-format   string (("en" "<p class=\"author\">Author: %a (%e)</p>
<p class=\"date\">Date: %d</p>
<p class=\"creator\">Generated by %c</p>
<p class=\"xhtml-validation\">%v</p>
"))
org-export-html-preamble-format   string (("en" ""))
org-export-html-toplevel-hlevel   string 2 (require 'org-export)
org-export-odt-content-template-file   file nil
org-get-priority-function   function nil
org-icalendar-combined-description   string nil
org-link-translation-function   function nil
org-make-link-description-function   function nil
org-mouse-1-follows-link   boolean 450
org-plantuml-jar-path   string nil
org-protocol-default-template-key   string nil
org-refile-target-verify-function   function nil
org-wl-namazu-default-index   directory nil
python-shell-virtualenv-path   string nil
reftex-format-label-function   function nil
shr-blocked-images   regexp nil
shr-color-visible-luminance-min   float 40
spam-report-resend-to   string nil
speechd-autospawn   string t
speechd-speak-separator-regexp   regexp nil
tls-certtool-program   string nil
type-break-good-break-interval   integer nil
uudecode-use-external   boolean "/usr/bin/uudecode"
verilog-active-low-regexp   string nil  (require 'verilog-mode)
verilog-auto-inout-ignore-regexp   string nil
verilog-auto-input-ignore-regexp   string nil
verilog-auto-output-ignore-regexp   string nil
verilog-auto-tieoff-ignore-regexp   string nil
verilog-auto-unused-ignore-regexp   string nil
verilog-typedef-regexp   string nil
viper-emacs-state-cursor-color   string nil
viper-search-scroll-threshold   boolean 2

dd
dictionary-tooltip-dictionary   string nil  (require 'dictionary)
feedmail-display-full-frame   boolean queued   (require 'feedmail)
jack-set-rtlimits-program   file nil
org-clock-file-time-cell-format   boolean "*%s*"
org-clock-total-time-cell-format   boolean "*%s*"
speechd-autospawn   string t
speechd-speak-movement-on-insertions   boolean read-only
truncate-partial-width-windows   boolean 50
viper-search-scroll-threshold   boolean 2


(require 'jack)
(symbol-plist 'jack-set-rtlimits-program)

(remove-hook 'minibuffer-setup-hook 'highlight-completion-setup)


;;-----------------------------------------------------------------------------
(ac-set-trigger-key "")
(ac-set-trigger-key nil)
(ac-set-trigger-key "C-O")
(setq ac-trigger-key "C-O")
(setq ac-trigger-key nil)
(customize-variable 'ac-trigger-key)

(progn
  (defvar my-keymap (make-sparse-keymap))
  (define-key my-keymap "" 'foo)
  (describe-variable 'my-keymap))

;;-----------------------------------------------------------------------------
(pp (symbol-plist 'texinfo) (current-buffer))
(custom-loads
 (informat "document" texinfo)
 group-documentation "Texinfo Mode." custom-links
 ((custom-group-link :tag "Font Lock Faces group" font-lock-faces))
 custom-group
 ((texinfo-open-quote custom-variable)
  (texinfo-close-quote custom-variable)
  (texinfo-mode-hook custom-variable)
  (texinfo-heading custom-face)
  (texinfo-texi2dvi-command custom-variable)
  (texinfo-tex-command custom-variable)
  (texinfo-texindex-command custom-variable)
  (texinfo-delete-from-print-queue-command custom-variable)
  (Info-split-threshold custom-variable)))



;;-----------------------------------------------------------------------------
(progn
  (require 'dictionary)
  (customize-variable 'dictionary-tooltip-dictionary))
/usr/share/emacs/site-lisp/dictionary-el/dictionary.el

(progn
  (setq dictionary-server "localhost")
  (dictionary-tooltip-mode 1))
(let ((dictionary-balloon-help-position (point)))
  'cat
  (dictionary-balloon-help-description))
(dictionary-definition "cat")


;;-----------------------------------------------------------------------------
(progn
  (require 'org-html)
  (customize-variable 'org-export-html-toplevel-hlevel))
