#!/usr/bin/perl -w

# Copyright 2015 Kevin Ryde.

# This file is part of miscbits-el.
#
# miscbits-el is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# miscbits-el is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with miscbits-el; see the file COPYING.  Failing that, go to
# <http://www.gnu.org/licenses/>.


# Look in the Debian "Contents" file for what packages have .el files.
#
# cf. python-kivy
#     tetex-brev  -- misc letter writing bits
#     recutils -- no site-start
#
#     nescc -- cc-mode derivative
#     hlint -- could autoload hs-lint, some unnecessary setf
#     /doc/gramadoir -- some program run

use strict;
use File::Slurp;
use IO::Uncompress::Gunzip;

# uncomment this to run the ### lines
# use Smart::Comments;


#-------------------------
use AptPkg::Config;
my $config = $AptPkg::Config::_config;
$config->init;
$config->{quiet} = 2;   # supress cache building messages
use AptPkg::Cache;
my $cache = AptPkg::Cache->new;
my $pkgrecords = $cache->packages;  # AptPkg::PkgRecords

sub package_name_to_shortdesc {
  my ($name) = @_;
  my $k = $pkgrecords->lookup($name);
  return $k->{'ShortDesc'};
}

#-------------------------
my $contents_filename = '/so/debian/Contents-i386.gz';
unless (-e '/tmp/Contents-by-package') {
  system("zcat /so/debian/Contents-i386.gz | sort --stable --key=2 --output=/tmp/Contents-by-package");
  system("ls -l /tmp/Contents-by-package");
}
$contents_filename = "/tmp/Contents-by-package";

#-------------------------

sub is_elfile {
  my ($filename) = @_;
  return ($filename =~ m{\.el(\.gz)?$}
          && $filename ne 'usr/share/aptitude/aptitude-defaults.el'
          && $filename !~ m{usr/share/vim/vim\d*/tutor/tutor.el}
          && $filename ne 'usr/share/transgui/lang/transgui.el'
          && $filename ne 'usr/share/perl5/auto/share/dist/Lingua-Sentence/nonbreaking_prefix.el'
          && $filename ne 'usr/share/mc/hints/mc.hint.el'
          && $filename ne 'usr/share/WindowMaker/menu.el'
         );
}
sub is_usual_elfile {
  my ($filename) = @_;
  return ($filename =~ m{^usr/share/x?emacs}
          || $filename =~ m{^etc/x?emacs/}
         );
}
sub any_site_start {
  my ($filenames) = @_;
  return ((grep { m{^etc/x?emacs/} } @$filenames)
          ? 1 : 0);
}

my @counts;
sub consider_files {
  my ($package, $filenames) = @_;
  ### $package
  ### filenames: scalar(@$filenames)
  return if $package =~ /^elpa-/;

  my $print = 0;
  my @reasons;

  my @elfiles = grep {is_elfile($_)} @$filenames;
  my @usuals;
  my @others;
  foreach my $filename (@elfiles) {
    if (is_usual_elfile($filename)) {
      push @usuals, $filename;
    } else {
      push @others, $filename;
    }
  }

  if (@usuals && ! any_site_start($filenames)) {
    push @reasons, "  no site-start\n";
    $print = 2;
  }
  if (@others) {
    push @reasons, "  elfiles in unusual places\n";
    @others = sort @others;
    foreach my $filename (@others) {
      push @reasons, "    $filename\n";
    }
    $print = 2;
  }
  if (@elfiles && ! $print) {
    # print "$package  $count\n";
    my $count = scalar(@elfiles);
    push @counts, "$package  $count\n";
  }

  if ($print) {
    my $shortdesc = package_name_to_shortdesc($package) // '';
    print "$package -- $shortdesc\n";
    print @reasons;
    if ($print >= 2) {
      @$filenames = sort @$filenames;
      foreach my $filename (@$filenames) {
        print "  $filename\n";
      }
    }
  }
}

open my $fh, '<', $contents_filename or die;
my $this_package = '';
my @this_filenames;
while (defined (my $line = readline $fh)) {
  $line =~ /(.*?)[ \t]+(.*)/ or next;
  my $filename = $1;
  my $package = $2;
  $package =~ s{.*/}{};

  if ($package ne $this_package) {
    if ($this_package ne '') {
      consider_files($this_package, \@this_filenames);
    }
    $this_package = $package;
    @this_filenames = ();
  }
  push @this_filenames, $filename;
}
if (@this_filenames) {
  consider_files($this_package, \@this_filenames);
}
print "\n";
print sort @counts;

# foreach my $package (sort keys %count) {
#   print "$package   $count{$package} $file{$package}\n";
# }
exit 0;
