#!/usr/bin/perl -w

# Copyright 2013, 2020 Kevin Ryde.

# This file is part of miscbits-el.
#
# miscbits-el is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# miscbits-el is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with miscbits-el; see the file COPYING.  Failing that, go to
# <http://www.gnu.org/licenses/>.


# /usr/share/info/gri.info-1.gz

use 5.010;
use strict;
use warnings;
use POSIX;
use Perl6::Slurp;
use File::Find::Rule;
use Regexp::Common 'Emacs','no_defaults';  # Regexp::Common::Emacs

use lib::abs '.';
use MyStuff;

my $verbose = 0;
my $dir = "/usr/share/info";
chdir $dir or die $!;

foreach my $filename (sort glob("$dir/*.info*")) {
  if ($verbose) { print "look at $filename\n"; }
  my $str = zslurp($filename) // next;

  # # search for formfeeds, except info esc ones
  # while ($str =~ /[^\037][\f]/g) {
  #   my $pos = $-[0];
  #   my ($line, $col) = MyStuff::pos_to_line_and_column ($str, $pos);
  #   print "$filename:$line:$col: ff\n";
  # }

  # search for non-existent [image]
  while ($str =~ /[\0][\b]\[image ([^\0]+)/g) {
    my $pos = $-[0];
    my $body = $1;
    $body =~ /src=\"([^"]+)\"/ or next;
    my $src = $1;
    next if -e $src;
  
    my ($line, $col) = MyStuff::pos_to_line_and_column ($str, $pos);
    print "$filename:$line:$col: $src\n";
    $src =~ s{^\./}{};
    system 'locate','--',$src or print "\n";
  }
}

exit 0;

sub zslurp {
  my ($filename) = @_;
  my $str;
  eval { $str = Perl6::Slurp::slurp(['gzip(auto)'=>1],$filename); 1 }
    or do {
      #         if ($! == POSIX::ENOENT) {
      #        }
      print "Cannot read $filename: $!\n";
      return;
    };
  return $str;
}
