;; shell-command doesn't return exit status ...


;; Copyright 2007, 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.




;;-----------------------------------------------------------------------------
;; xemacs incompatibilities

;; the definitions offered here don't pretend to be full implementations,
;; but begin eval-when-compile macros they don't end up in the final .elc
(eval-when-compile
  (unless (fboundp 'replace-regexp-in-string)
    (defmacro replace-regexp-in-string (regexp repl str
                                               &optional fixedcase literal)
      `(progn
         (or ,fixedcase
             (error "replace-regexp-in-string: oops, only fixedcase"))
         (replace-in-string ,str ,regexp ,repl ,literal))))
  (unless (fboundp 'make-temp-file)
    (defmacro make-temp-file (prefix &optional dirflag)
      `(progn
         (eval-and-compile ;; at compile time too to quieten byte compiler
           (require 'mm-util)) ;; from gnus
         (mm-make-temp-file ,prefix ,dirflag))))
  (unless (fboundp 'set-buffer-multibyte)
    (defmacro set-buffer-multibyte (flag)
      flag))
  (unless (fboundp 'display-images-p)
    (defmacro display-images-p ()
      'window-system))
  (unless (fboundp 'image-type-available-p)
    (defmacro image-type-available-p (type)
      `(memq ,type (image-instantiator-format-list))))
  (unless (fboundp 'insert-image)
    (defmacro insert-image (image-spec string)
      `(let ((spec (copy-sequence (cdr ,image-spec))))
         (insert (or ,string " "))
         (let* ((type   (plist-get spec :type))
                (glyph  (make-glyph (apply 'vector type
                                           (plist-remprop spec :type))))
                (extent (make-extent (1- (point)) (point))))
           (set-extent-property extent 'end-glyph glyph))))))


