;; Copyright 2010, 2013 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)

  (let ((count 0)
        (case-fold-search nil)
        (tempbuf "*t-checkdoc-url-buttonize.el temp*"))
    (switch-to-buffer tempbuf)
    (erase-buffer)

    ;;-------------------------------------------------------------------------
    ;; autoloads

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(checkdoc-url-buttonize-docstring))

    (mapcar (lambda (elem)
              (let* ((hook (nth 0 elem))
                     (func (nth 1 elem))
                     (options (get hook 'custom-options)))
                (unless (and (listp options)
                             (memq func options))
                  (error "`%S' not an option on `%S'" func hook)))
              (setq count (1+ count)))

            '((checkdoc-style-hooks checkdoc-url-buttonize-docstring)))


    ;;-------------------------------------------------------------------------
    ;; checkdoc-url-buttonize-docstring

    ;; must load before let-binding `checkdoc-autofix-flag' and
    ;; `checkdoc-style-hooks'
    (require 'checkdoc)

    (let ((checkdoc-autofix-flag 'automatic)
          (checkdoc-style-hooks '(checkdoc-url-buttonize-docstring)))
      (mapcar (lambda (elem)
                (let ((input (nth 0 elem))
                      (want  (nth 1 elem)))
                  (erase-buffer)
                  (insert input)
                  (goto-char (point-min))
                  (end-of-line)
                  (let ((got (condition-case err
                                 (progn
                                   (checkdoc-defun)
                                   (buffer-string))
                               (error (format "error: %s"
                                              (error-message-string err))))))
                    (unless (equal want got)
                      (goto-char (point-max))
                      (erase-buffer)
                      (insert "\n---input---\n")
                      (insert input)
                      (insert "\n---got---\n")
                      (insert (format "%s\n" got))
                      (insert "\n---want---\n")
                      (insert want)
                      (error "wrong at %s\n%s\n" count (buffer-string)))))
                (setq count (1+ count)))

              '(

;;; ---------------------
;;; http
                ("\
\(defun foo ()
  \"Foo bar.
http://foo.org\"
  nil)
" "\
\(defun foo ()
  \"Foo bar.
URL `http://foo.org'\"
  nil)
")

;;; ---------------------
;;; multiple schemes
                ("\
\(defun foo ()
  \"Foo bar.
http://foo.org
http://bar.org
https://quux.org
ftp://foo.org
sftp://bar.org
news://alt.religion.emacs\"
  nil)
" "\
\(defun foo ()
  \"Foo bar.
URL `http://foo.org'
URL `http://bar.org'
URL `https://quux.org'
URL `ftp://foo.org'
URL `sftp://bar.org'
news://alt.religion.emacs\"
  nil)
")

;;; ---------------------
;;; sample http://... left alone
                ("\
\(defun foo ()
  \"Foo bar.
http://... blah blah.\"
  nil)
" "\
\(defun foo ()
  \"Foo bar.
http://... blah blah.\"
  nil)
")
                )))

    ;;-------------------------------------------------------------------------
    ;; help-xref-url-regexp availability

    (require 'checkdoc-url-buttonize)
    (message "help-xref-url-regexp in this emacs: %S"
             (boundp 'help-xref-url-regexp))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (unload-feature 'checkdoc-url-buttonize t)

    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))
