;; Copyright 2006, 2007, 2009, 2010 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (let ((case-fold-search t)
        (count 0)
        (tempbuf "*t-ffap-makefile-vars--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'ffap-makefile-vars)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; ffap-makefile-vars-find

    (mapcar (lambda (elem)
              (let ((name  (nth 0 elem))
                    (str   (nth 1 elem))
                    (want (nth 2 elem)))
                (erase-buffer)
                (insert str)
                (goto-char (point-min))
                (let ((got (ffap-makefile-vars-find name)))
                  (unless (equal got want)
                    (goto-char (point-max))
                    (insert "\n\n---name---\n")
                    (insert name)
                    (insert "\n---str---\n")
                    (insert str)
                    (insert "\n---want---\n")
                    (insert want)
                    (insert "\n---got---\n")
                    (insert got)
                    (insert "\n--------\n")
                    (error "got: %S\nwant: %S\nat count %s"
                           got want count))))
              (setq count (1+ count)))

            '(("FOO" "FOO=123" "123")
              ("FOO" "FOO = 123\nBAR = 456" "123")
              ("FOO" "FOO = 123 \\\n456" "123 456")
              ("FOO" "\nFOO = 123 \\\n456" "123 456")
              ("FOO" "\n FOO = 123 \\\n456" "123 456")

              ("FOO" "FOO=123\nfoo=456" "123")
              ("foo" "FOO=123\nfoo=456" "456")
              ("FOO" "foo=456\nFOO=123\n" "123")
              ("foo" "foo=456\nFOO=123\n" "456")

              ("FOO" "\
FOO = 123 \\
FOO = 456"
               "123 FOO = 456")
              ("FOO" "\
FOO=123 \\
FOO=456 \\
FOO=789"
               "123 FOO=456 FOO=789")
              ("FOO" "\
FOO = 999
FOO = 123 \\
FOO = 456"
               "123 FOO = 456")))

    ;;-------------------------------------------------------------------------
    ;; ffap-makefile-vars-substitute

    (dolist (elem '(("$(FOO)" "" "")
                    ("$(FOO)" "FOO=123" "123")
                    ("x$(FOO)" "FOO=123" "x123")
                    ("x$(FOO)y" "FOO=123" "x123y")
                    ("$(FOO)$(FOO)" "FOO=123" "123123")
                    ("$$(FOO)" "FOO=123" "$(FOO)")
                    ("$$$(FOO)" "FOO=123" "$123")

                    ("$(FOO)" "FOO=$(BAR)\nBAR=123" "123")
                    ("$(FOO)" "FOO=x$(BAR)\nBAR=123" "x123")
                    ("$(FOO)" "FOO=$(BAR)x\nBAR=123" "123x")
                    ("$(FOO)" "FOO=$(BAR)$(BAR)x\nBAR=123" "123123x")

                    ;; recursively
                    ("$(FOO)" "\
FOO=$(BAR)$(BAR)$(QUUX)
BAR=$(QUUX)$$$(QUUX)
QUUX=$(XYZZY)
XYZZY=k" "k$kk$kk")

                    ;; indirection
                    ("$(FOO)" "\
XYZ=123
ABC=456
BAR=XYZ
FOO=$($(BAR))" "123")

                    ;; circular
                    ("$(FOO)" "FOO=$(FOO)" "$(FOO)")
                    ("$(FOO)" "FOO=x$(FOO)" "$(FOO)")
                    ("$(FOO)" "
FOO=x$(BAR)
BAR=y$(FOO)" "$(FOO)")
                    ))

      (let ((str  (nth 0 elem))
            (buf  (nth 1 elem))
            (want (nth 2 elem)))
        (erase-buffer)
        (insert buf)
        (let ((got (ffap-makefile-vars-substitute str)))
          (unless (equal got want)
            (goto-char (point-max))
            (insert "\n---\nffap-makefile-vars-substitute wrong\n")
            (insert "---str---\n")
            (insert str)
            (insert "\n---want---\n")
            (insert want)
            (insert "\n---got---\n")
            (insert got)
            (insert "\n--------\n")
            (error "got: %S\nwant: %S\nat count %s"
                   got want count))))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'ffap-makefile-vars))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))))


    ;;-------------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'ffap-makefile-vars))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;; ------------------------------------------------------------------------
    (bury-buffer)
    (message "  %d tests" count)
    count))
