;; This was an idea for the byte compiler to notice (concat str "aa" "bb")
;; and combine adjacent fixed strings like (concat str "aabb").  Quite good
;; for multi-line inserts or possibly for regexps constructed from pieces.
;; Don't think it ever really worked though.



;;; byte-opt-concat.el --- optimize adjacent string arguments

;; Version: 1
;; Keywords: lisp, byte-compile

;; Copyright 2009, 2010, 2011, 2016 Kevin Ryde

;; byte-opt-concat.el is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as published
;; by the Free Software Foundation; either version 3, or (at your option)
;; any later version.
;;
;; byte-opt-concat.el is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You can get a copy of the GNU General Public License online at
;; <http://www.gnu.org/licenses/>.


;;; Code:

(defun byte-optimize-pairwise (form pred reduce &optional reduce2)
  ;; If adjacent args both satisfy (funcall PRED x) then collapse them to
  ;; (funcall REDUCE x y).  If no adjacent args satisfy PRED then return
  ;; FORM unchanged.
  ;;
  ;; If REDUCE2 is not nil then REDUCE is used when x is the first argument,
  ;; then REDUCE2 is used for subsequent args.
  ;;
  (let ((ret     (list (car form)))
	(arglist (cdr form))
	changed)
    (while arglist
      (let ((arg (pop arglist)))
        (if (funcall pred arg)
            (while (and arglist (funcall pred (car arglist)))
              (setq arg (funcall reduce arg (pop arglist)))
              (setq changed t)))
	(push arg ret)
        (if reduce2 (setq reduce reduce2))))

    (if changed
        (nreverse ret)
      form)))

(put 'concat 'byte-optimizer 'byte-optimize-concat)
(defun byte-optimize-concat (form)
  ;; (concat "x" "y" (foo)) -> (concat "xy" (foo))
  (byte-optimize-pairwise 'stringp 'concat))

(put 'insert                            'byte-optimizer 'byte-optimize-insert)
(put 'insert-and-inherit                'byte-optimizer 'byte-optimize-insert)
(put 'insert-before-markers             'byte-optimizer 'byte-optimize-insert)
(put 'insert-before-markers-and-inherit 'byte-optimizer 'byte-optimize-insert)
(defun byte-optimize-insert (form)
  ;; (insert "x" "y" ?z (foo)) -> (insert "xy" ?z (foo))
  (setq form (byte-optimize-concat form))

  ;; (insert "x") -> (insert ?x) since a char is less memory.
  ;; But only for ascii, in case high char values change again they way they
  ;; did for unicode in Emacs 23 (strings in a .elc are decoded according to
  ;; the file charset, plain numbers are not).
  (when (and (= 2 (length form))
             (stringp (cadr form))
             (= 1 (length (cadr form)))
             (string-match "[[:ascii:]]" (cadr form)))
    (setq form (list (car form)
                     (string-to-char (cadr form)))))
  form)

(put 'princ-list 'byte-optimizer 'byte-optimize-princ-list)
(defun byte-optimize-princ-list (form)
  ;; (princ-list "x" "y") -> (princ-list "xy")
  (setq form (byte-optimize-concat form))

  ;; (princ-list "") -> (terpri)
  (if (equal (cdr form) '(""))
      (setq form (list (car form))))

  ;; as per defsubst
  (byte-compile-inline-expand form))



(put '+ 'byte-optimizer 'byte-optimize-+)
(defun byte-optimize-+ (form)
  ;; (+ 1 2 (foo)) -> (+ 3 (foo))
  (setq form (byte-optimize-pairwise form 'numberp '+))
  (byte-optimize-plus form))

(put '- 'byte-optimizer 'byte-optimize--)
(defun byte-optimize-- (form)
  ;; (- 1 2 3 (foo) 4 5) -> (- -4 (foo) 9)
  (setq form (byte-optimize-pairwise form 'numberp '- '+))

  (byte-optimize-minus form))

(provide 'byte-opt-concat)

;;; byte-opt-concat.el ends here
